from typing import Any
from jwt import decode
import logging

from authlib.integrations.starlette_client.apps import StarletteOAuth2App

from .settings import AuthProviderSettings, settings

logger = logging.getLogger("oidc-test")


class Provider(AuthProviderSettings):
    class Config:
        arbitrary_types_allowed = True

    authlib_client: StarletteOAuth2App = StarletteOAuth2App(None)

    def decode(self, token: str, verify_signature: bool = True) -> dict[str, Any]:
        """Decode the token with signature check"""
        if settings.debug_token:
            decoded = decode(
                token,
                self.get_public_key(),
                algorithms=[self.signature_alg],
                audience=["account", "oidc-test", "oidc-test-web"],
                options={
                    "verify_signature": False,
                    "verify_aud": False,
                },  # not settings.insecure.skip_verify_signature},
            )
            logger.debug(str(decoded))
        return decode(
            token,
            self.get_public_key(),
            algorithms=[self.signature_alg],
            audience=["account", "oidc-test", "oidc-test-web"],
            options={
                "verify_signature": verify_signature,
            },  # not settings.insecure.skip_verify_signature},
        )


providers: dict[str, Provider] = {}
